# -*- coding: utf-8 -*-
"""Art modules."""
from .art import artError
from .art import aprint, art, randart
from .art import tprint, tsave, text2art
from .art import decor
from .art import get_font_dic, set_default, help_func, art_list, font_list, decor_list
from .art_param import ART_VERSION, FONT_NAMES, ART_NAMES, DECORATION_NAMES, DEFAULT_FONT
from .art_param import ART_COUNTER, FONT_COUNTER, DECORATION_COUNTER
from .art_param import NON_ASCII_ARTS, NON_ASCII_FONTS, ASCII_FONTS, ASCII_ARTS
__version__ = ART_VERSION



# -*- coding: utf-8 -*-
"""Art and decors duplication and UTF-8 compatibility testing script."""
import sys
import art
from art.art_param import DECORATIONS_MAP as Decor_Dict
from art.art_dic import art_dic as Art_Dict
from art.art_param import NON_ASCII_ARTS
from font_wizard import is_utf8, is_ascii

Failed1 = 0
Failed2 = 0
Failed3 = 0
Failed4 = 0
Message1 = "Art UTF-8 compatibility test "
Message2 = "Art duplication test "
Message3 = "Decor UTF-8 compatibility test "
Message4 = "Decor duplication test "


def print_result(flag_list, message_list):
    """
    Print final result.

    :param flag_list: list of test flags
    :type flag_list: list
    :param message_list: list of test messages
    :type message_list: list
    :return: None
    """
    print("Art version : {}".format(art.__version__))
    print("Number of arts : {}".format(art.ART_COUNTER))
    print("Number of decors : {}".format(art.DECORATION_COUNTER))
    for index, flag in enumerate(flag_list):
        if flag == 0:
            print(message_list[index] + "passed!")
        else:
            print(message_list[index] + "failed!")


if __name__ == "__main__":
    Art_Keys = list(Art_Dict)
    Art_Values = list(Art_Dict.values())
    for art_name in Art_Keys:
        ascii_flag = is_ascii(Art_Dict[art_name])
        if not is_utf8(Art_Dict[art_name]):
            Failed1 += 1
            print("UTF-8 compatibility error in art : " + art_name)
        if Art_Values.count(Art_Dict[art_name]) > 1:
            Failed2 += 1
            print("Art duplication error : " + art_name)
        if ascii_flag and art_name in NON_ASCII_ARTS:
            print(
                "Art type warning : {0} is ASCII but imported as NON-ASCII".format(art_name))
        if not ascii_flag and art_name not in NON_ASCII_ARTS:
            print(
                "Art type warning : {0} is NON-ASCII but imported as ASCII".format(art_name))

    Decor_Keys = list(Decor_Dict)
    Decor_Values = list(Decor_Dict.values())
    for decor_name in Decor_Keys:
        if not is_utf8(
                Decor_Dict[decor_name][0]) or not is_utf8(
                Decor_Dict[decor_name][1]):
            Failed3 += 1
            print("UTF-8 compatibility error in decor : " + decor_name)
        if Decor_Values.count(Decor_Dict[decor_name]) > 1:
            Failed4 += 1
            print("Decor duplication error : " + decor_name)

    print_result([Failed1, Failed2, Failed3, Failed4], [
                 Message1, Message2, Message3, Message4])
    sys.exit(Failed1 + Failed2 + Failed3 + Failed4)



# -*- coding: utf-8 -*-
"""Notebook-check script."""
import os
import nbformat
from nbconvert.preprocessors import ExecutePreprocessor
from art import tprint

NOTEBOOKS_LIST = [
    "ArtList",
    "DecorList",
    "FontList"]

EXTENSION = ".ipynb"

if __name__ == "__main__":
    tprint("ART", "bulbhead")
    tprint("Notebook Check", "bulbhead")
    print("Processing ...")
    for index, notebook in enumerate(NOTEBOOKS_LIST):
        ep = ExecutePreprocessor(timeout=6000, kernel_name='python3')
        path = os.path.join("", notebook)
        with open(path + EXTENSION, "r", encoding="utf-8") as f:
            nb = nbformat.read(f, as_version=4)
            ep.preprocess(nb, {'metadata': {'path': ''}})
        with open(path + EXTENSION, 'w', encoding='utf-8') as f:
            nbformat.write(nb, f)
        print("{0}.{1} [OK]".format(str(index + 1), notebook))



# -*- coding: utf-8 -*-
"""Version-check script."""
import os
import sys
import codecs
from art.art_param import *

Failed = 0
VERSION = "6.1"

README_ITEMS = ['<td id="font_counter">{0}</td>'.format(str(FONT_COUNTER)),
                '<img src="https://img.shields.io/badge/Font List-{0}-blue.svg" alt="Font List">'.format(str(FONT_COUNTER)),
                '<td id="art_counter">{0}</td>'.format(str(ART_COUNTER)),
                '<img src="https://img.shields.io/badge/Art List-{0}-orange.svg" alt="Art List">'.format(str(ART_COUNTER)),
                '<td id="decor_counter">{0}</td>'.format(str(DECORATION_COUNTER)),
                '<img src="https://img.shields.io/badge/Decor List-{0}-green.svg" alt="Decor List">'.format(str(DECORATION_COUNTER))]

SETUP_ITEMS = [
    "version='{0}'"]
INSTALL_ITEMS = [
    "[Version {0}](https://github.com/sepandhaghighi/art/archive/v{0}.zip)",
    "pip install art=={0}"]
CHANGELOG_ITEMS = [
    "## [{0}]",
    "https://github.com/sepandhaghighi/art/compare/v{0}...dev",
    "[{0}]:"]
ART_LIST_ITEMS = ["### Version : {0}"]
FONT_LIST_ITEMS = ["### Version : {0}"]
PARAMS_ITEMS = ['ART_VERSION = "{0}"']
META_ITEMS = ['% set version = "{0}" %']

FILES = {
    "setup.py": SETUP_ITEMS,
    "INSTALL.md": INSTALL_ITEMS,
    "CHANGELOG.md": CHANGELOG_ITEMS,
    "FontList.ipynb": FONT_LIST_ITEMS,
    "ArtList.ipynb": ART_LIST_ITEMS,
    os.path.join(
        "art",
        "art_param.py"): PARAMS_ITEMS,
    os.path.join("otherfile", "meta.yaml"): META_ITEMS}

TEST_NUMBER = len(FILES) + 1


def print_result(failed=False):
    """
    Print final result.

    :param failed: failed flag
    :type failed: bool
    :return: None
    """
    message = "Version/Counter tag tests "
    if not failed:
        print("\n" + message + "passed!")
    else:
        print("\n" + message + "failed!")
    print("Passed : " + str(TEST_NUMBER - Failed) + "/" + str(TEST_NUMBER))


if __name__ == "__main__":
    for file_name, test_items in FILES.items():
        try:
            file_content = codecs.open(
                file_name, "r", "utf-8", "ignore").read()
            for test_item in test_items:
                if file_content.find(test_item.format(VERSION)) == -1:
                    print("Incorrect version tag in " + file_name)
                    Failed += 1
                    break
        except Exception as e:
            Failed += 1
            print("Error in " + file_name + "\n" + "Message : " + str(e))

    try:
        readme_file_content = codecs.open(
            "README.md", "r", "utf-8", "ignore").read()
        for test_item in README_ITEMS:
            if readme_file_content.find(
                    test_item) == -1:
                print("Incorrect counter in " + "README.md")
                Failed += 1
                break
    except Exception as e:
        Failed += 1
        print("Error in README.md" + "\n" + "Message : " + str(e))
    if Failed == 0:
        print_result(False)
        sys.exit(0)
    else:
        print_result(True)
        sys.exit(1)



# -*- coding: utf-8 -*-
"""Wizard script for new fonts."""
import string
import sys
import ast
import art
import json

Letters = string.ascii_letters + string.punctuation + string.digits
Font_List = list(art.art_param.FONT_MAP)

INVALID_FONT_NAME = [
    "mix",
    "wizard",
    "wiz",
    "magic",
    "random",
    "random-na",
    "random-xlarge",
    "random-large",
    "random-medium",
    "random-small",
    "rand",
    "rand-na",
    "rand-xlarge",
    "rand-large",
    "rand-medium",
    "rand-small",
    "rnd",
    "rnd-na",
    "rnd-xlarge",
    "rnd-large",
    "rnd-medium",
    "rnd-small"]

Error1 = "[Error] Font data is empty!"
Error2 = "[Error] Font should support 95 printable ASCII characters, please check font data!"
Error3 = "[Error] Font duplication (art version : {}) -- > ".format(
    art.__version__)
Error4 = "[Error] All letters should have same height"
Error5 = "[Error] Font should be compatible with UTF-8"
Error6 = "[Error] This font name is not available"
Error7 = "[Error] All lines in a letter should have same width (letter : {0})"


def is_utf8(s):
    """
    Check input string for UTF-8 compatibility.

    :param s: input string
    :type s: str
    :return: result as bool
    """
    try:
        if sys.version_info.major == 3:
            _ = bytes(s, 'utf-8').decode('utf-8', 'strict')
        else:
            return True
        return True
    except Exception:
        return False


def is_ascii(s):
    """
    Check input string for ASCII compatibility.

    :param s: input string
    :type s: str
    :return: result as bool
    """
    for i in s:
        if ord(i) > 127:
            return False
    return True


if __name__ == "__main__":
    art.tprint("Font Wizard")
    print("Use this string as input for font resource : ")
    print(Letters)
    print("*" * 30)
    while True:
        font_name = input("Please enter font name : ")
        if font_name in Font_List or font_name.lower() in INVALID_FONT_NAME:
            print(Error6)
        else:
            break
    font_data = input("Please enter font data (string or list) : ")
    if not is_utf8(font_data):
        print(Error5)
        sys.exit()
    ascii_flag = "ASCII" if is_ascii(font_data) else "Non-ASCII"
    if not font_data:
        print(Error1)
        sys.exit()
    if len(font_data) != len(Letters):
        try:
            font_data = ast.literal_eval(font_data)
            if len(font_data) != len(Letters):
                print(Error2)
                sys.exit()
        except Exception:
            print(Error2)
            sys.exit()
    font_dic = dict(zip(Letters, font_data))
    font_height_list = [len(x.split("\n")) for x in font_dic.values()]
    if len(set(font_height_list)) != 1:
        print(Error4)
        sys.exit()
    font_height = font_height_list[0]
    if " " not in font_dic:
        _ = [" "] * font_height
        font_dic[" "] = "\n".join(_)
    for font2 in Font_List:
        if len(font_dic) == len(art.get_font_dic(font2)):
            if font_dic == art.get_font_dic(font2):
                print(Error3 + font2)
                sys.exit()
        else:
            font1_keys = set(font_dic)
            font2_keys = set(art.get_font_dic(font2))
            inter_keys = list(font1_keys.intersection(font2_keys))
            font1_map = []
            font2_map = []
            for letter in inter_keys:
                font1_map.append(font_dic[letter])
                font2_map.append(art.get_font_dic(font2)[letter])
            if font1_map == font2_map:
                print(Error3 + font2)
                sys.exit()
    first_line_list = [x.split("\n")[0] in ["", " "] for x in font_dic.values()]
    last_line_list = [x.split("\n")[-1] in ["", " "] for x in font_dic.values()]
    for letter, letter_data in font_dic.items():
        letter_data_split = letter_data.split("\n")
        width_list = [len(x) for x in letter_data_split]
        if letter_data_split[-1] in ["", " "] and all(last_line_list):
            width_list = width_list[:-1]
        if width_list and letter_data_split[0] in ["", " "] and all(first_line_list):
            width_list = width_list[1:]
        if len(set(width_list)) > 1:
            print(Error7.format(letter))
            sys.exit()
    if len(font_dic) == 95:
        print("Done!")
        print("Font dictionary : \n")
        print(
            "{}_dic =".format(font_name),
            json.dumps(
                font_dic,
                ensure_ascii=False,
                indent=4))
        print("- Add this dictionary to the end of `text_dic3.py`")
        print("- This font is {0} : ".format(ascii_flag))
        print('\t1. Add "{0}": [{0}_dic, False] to the end of `FONT_MAP` dictionary in `art_param.py`'.format(
            font_name))
        if ascii_flag == "ASCII":
            print("\t2. Add a new test case to `test.py`")
        else:
            print("\t2. Add a new test case to `test2.py`")
            print('\t3. Add "{0}" to the end of `NON_ASCII_FONTS` list in `art_param.py`'.format(
                font_name))
            if font_height > 1:
                print('\t4. Add "{0}" to the end of `MIX_FILTERED_FONTS` list in `art_param.py`'.format(
                    font_name))
    else:
        print(Error2)



# -*- coding: utf-8 -*-
"""Art module."""
from .art_dic import *
from .art_param import *
import os
import random


class artError(Exception):  # pragma: no cover
    """Art error class."""

    pass


def font_size_splitter(font_map):
    """
    Split fonts to 4 category (small,medium,large,xlarge) by maximum length of letter in each font.

    :param font_map: input fontmap
    :type font_map : dict
    :return: splitted fonts as dict
    """
    small_font, medium_font, large_font, xlarge_font = [], [], [], []
    fonts = set(font_map) - set(RANDOM_FILTERED_FONTS)
    for font in sorted(fonts):
        length = max(len(x) for x in font_map[font][0].values())
        if length <= FONT_SMALL_THRESHOLD:
            small_font.append(font)
        elif length <= FONT_MEDIUM_THRESHOLD:
            medium_font.append(font)
        elif length <= FONT_LARGE_THRESHOLD:
            large_font.append(font)
        else:
            xlarge_font.append(font)
    return {
        "small_list": small_font,
        "medium_list": medium_font,
        "large_list": large_font,
        "xlarge_list": xlarge_font}


RND_SIZE_DICT = font_size_splitter(FONT_MAP)  # pragma: no cover


def line(char="*", number=30):
    """
    Print line of chars.

    :param char: input character
    :type char:str
    :param number: number of characters
    :return: None
    """
    print(char * number)


def font_list(text="test", mode="all"):
    """
    Print all fonts.

    :param text : input text
    :type text : str
    :param mode: fonts mode (all,ascii,non-ascii)
    :type mode: str
    :return: None
    """
    fonts = set(FONT_NAMES)
    if mode.lower() == "ascii":
        fonts = fonts - set(NON_ASCII_FONTS)
    if mode.lower() == "non-ascii":
        fonts = set(NON_ASCII_FONTS)
    for item in sorted(fonts):
        print(str(item) + " : ")
        text_temp = text + "\n"
        tprint(text_temp, str(item))


def art_list(mode="all"):
    """
    Print all 1-Line arts.

    :param mode: fonts mode (all,ascii,non-ascii)
    :type mode: str
    :return: None
    """
    arts = set(ART_NAMES)
    if mode.lower() == "ascii":
        arts = arts - set(NON_ASCII_ARTS)
    if mode.lower() == "non-ascii":
        arts = set(NON_ASCII_ARTS)
    for i in sorted(arts):
        print(i)
        aprint(i)
        line()


def decor_list(text="test", font="fancy6"):
    """
    Print all decorations.

    :param text : input text
    :type text : str
    :param font: input font
    :type font:str
    :return: None
    """
    for decor in DECORATION_NAMES:
        print(decor)
        tprint(text, font=font, decoration=decor)
        line()


def help_func():
    """
    Print help page.

    :return: None
    """
    tprint("art")
    tprint("v" + ART_VERSION)
    print(DESCRIPTION)
    print(CLI_HELP)


def aprint(artname, number=1, space=1):
    """
    Print 1-line art.

    :param artname: art name
    :type artname : str
    :param number: number of repeats
    :type number: int
    :param space: space between arts
    :type space: int
    :return: None
    """
    try:
        if artname == "UnicodeEncodeError":
            raise UnicodeEncodeError(
                'test', u"", 42, 43, 'test unicode-encode-error')
        result, artname = art(
            artname=artname, number=number, space=space, __detailed_return=True)
        print(result)
    except UnicodeEncodeError:
        print(ART_ENVIRONMENT_WARNING.format(artname))


def art(artname, number=1, space=1, __detailed_return=False):
    """
    Return 1-line art.

    :param artname: art name
    :type artname : str
    :param number: number of repeats
    :type number: int
    :param space: space between arts
    :type space: int
    :param __detailed_return: flag for returning the art name
    :type __detailed_return: bool
    :return: ascii art as str
    """
    if not isinstance(artname, str):
        raise artError(ART_TYPE_ERROR)
    if not isinstance(number, int):
        raise artError(NUMBER_TYPE_ERROR)
    if not isinstance(space, int):
        raise artError(SPACE_TYPE_ERROR)
    artname = artname.lower()
    arts = ART_NAMES
    if artname in ["random", "rand", "rnd"]:
        filtered_arts = list(set(arts) - set(RANDOM_FILTERED_ARTS))
        artname = random.choice(filtered_arts)
    elif artname not in art_dic:
        selected_art = min(arts, key=lambda x: distance_calc(artname, x))
        min_distance = distance_calc(artname, selected_art)
        threshold = max(len(artname), len(selected_art)) / 2
        if min_distance < threshold:
            artname = selected_art
        else:
            raise artError(ART_NAME_ERROR)
    art_value = art_dic[artname]
    result = (art_value + " " * space) * number
    result = result.strip()
    if __detailed_return:
        return (result, artname)
    return result


def randart():
    """
    Return random 1-line art.

    :return: ascii art as str
    """
    return art("random")


def tprint(
        text,
        font=DEFAULT_FONT,
        chr_ignore=True,
        decoration=None,
        sep="\n",
        space=0):
    r"""
    Print art text (support \n).

    :param text: input text
    :type text:str
    :param font: input font
    :type font:str
    :param chr_ignore: ignore not supported character
    :type chr_ignore:bool
    :param decoration: text decoration
    :type decoration:str
    :param sep: line separator char
    :type sep: str
    :param space: space between characters
    :type space: int
    :return: None
    """
    try:
        if font == "UnicodeEncodeError":
            raise UnicodeEncodeError(
                'test', u"", 42, 43, 'test unicode-encode-error')
        result, font, decoration = text2art(
            text,
            font=font,
            decoration=decoration,
            chr_ignore=chr_ignore,
            sep=sep,
            space=space,
            __detailed_return=True)
        print(result)
    except UnicodeEncodeError:
        if decoration is not None:
            print(FONT_OR_DECOR_ENVIRONMENT_WARNING.format(font, decoration))
        else:
            print(FONT_ENVIRONMENT_WARNING.format(font))


def tsave(
        text,
        font=DEFAULT_FONT,
        filename="art",
        chr_ignore=True,
        print_status=True,
        overwrite=False,
        decoration=None,
        sep="\n",
        space=0):
    r"""
    Save ascii art (support \n).

    :param text: input text
    :param font: input font
    :type font:str
    :type text:str
    :param filename: output file name
    :type filename:str
    :param chr_ignore: ignore not supported character
    :type chr_ignore:bool
    :param print_status : save message print flag
    :type print_status:bool
    :param overwrite : overwrite the saved file if true
    :type overwrite:bool
    :param decoration: text decoration
    :type decoration:str
    :param sep: line separator char
    :type sep: str
    :param space: space between characters
    :type space: int
    :return: None
    """
    try:
        if not isinstance(text, str):
            raise Exception(TEXT_TYPE_ERROR)
        files_list = os.listdir(os.getcwd())
        splitted_filename = filename.split(".")
        if len(splitted_filename) > 1:
            name = filename[:-1 * filename[::-1].find('.') - 1]
            extension = "." + splitted_filename[-1]
        else:
            name = filename
            extension = ".txt"
        index = 2
        test_name = name
        while not overwrite:
            if test_name + extension in files_list:
                test_name = name + str(index)
                index = index + 1
            else:
                break
        file = open(test_name + extension, "w", encoding='utf-8')
        result = text2art(
            text,
            font=font,
            decoration=decoration,
            chr_ignore=chr_ignore,
            sep=sep,
            space=space)
        file.write(result)
        file.close()
        if print_status:
            print("Saved! \nFilename: " + test_name + extension)
        return {"Status": True, "Message": "OK"}
    except Exception as e:
        return {"Status": False, "Message": str(e)}


def distance_calc(s1, s2):
    """
    Calculate Levenshtein distance between two words.

    :param s1: first word
    :type s1 : str
    :param s2: second word
    :type s2 : str
    :return: distance between two word

    References :
    1- https://stackoverflow.com/questions/2460177/edit-distance-in-python
    2- https://en.wikipedia.org/wiki/Levenshtein_distance
    """
    if len(s1) > len(s2):
        s1, s2 = s2, s1

    distances = range(len(s1) + 1)
    for i2, c2 in enumerate(s2):
        distances_ = [i2 + 1]
        for i1, c1 in enumerate(s1):
            if c1 == c2:
                distances_.append(distances[i1])
            else:
                distances_.append(
                    1 + min((distances[i1], distances[i1 + 1], distances_[-1])))
        distances = distances_
    return distances[-1]


def wizard_font(text):
    """
    Check input text length for wizard mode.

    :param text: input text
    :type text:str
    :return: font as str
    """
    text_length = len(text)
    if text_length <= TEXT_XLARGE_THRESHOLD:
        font = random.choice(XLARGE_WIZARD_FONT)
    elif text_length <= TEXT_LARGE_THRESHOLD:
        font = random.choice(LARGE_WIZARD_FONT)
    elif text_length <= TEXT_MEDIUM_THRESHOLD:
        font = random.choice(MEDIUM_WIZARD_FONT)
    else:
        font = random.choice(SMALL_WIZARD_FONT)
    return font


def indirect_font(font, text):
    """
    Check input font for indirect modes.

    :param font: input font
    :type font : str
    :param text: input text
    :type text:str
    :return: font as str
    """
    fonts = FONT_NAMES
    if font in ["rnd-small", "random-small", "rand-small"]:
        font = random.choice(RND_SIZE_DICT["small_list"])
        return font
    if font in ["rnd-medium", "random-medium", "rand-medium"]:
        font = random.choice(RND_SIZE_DICT["medium_list"])
        return font
    if font in ["rnd-large", "random-large", "rand-large"]:
        font = random.choice(RND_SIZE_DICT["large_list"])
        return font
    if font in ["rnd-xlarge", "random-xlarge", "rand-xlarge"]:
        font = random.choice(RND_SIZE_DICT["xlarge_list"])
        return font
    if font in ["random", "rand", "rnd"]:
        filtered_fonts = list(set(fonts) - set(RANDOM_FILTERED_FONTS))
        font = random.choice(filtered_fonts)
        return font
    if font in ["wizard", "wiz", "magic"]:
        font = wizard_font(text)
        return font
    if font in ["rnd-na", "random-na", "rand-na"]:
        font = random.choice(NON_ASCII_FONTS)
        return font
    if font not in fonts:
        font = min(fonts, key=lambda x: distance_calc(font, x))
    return font


def indirect_decoration(decoration):
    """
    Check input decoration for indirect modes.

    :param decoration: input decoration
    :type decoration : str
    :return: decoration as str
    """
    decorations = DECORATION_NAMES
    if decoration in ["random", "rand", "rnd"]:
        decoration = random.choice(decorations)
        return decoration
    if decoration not in decorations:
        decoration = min(decorations, key=lambda x: distance_calc(decoration, x))
    return decoration


def mix_letters():
    """
    Return letters list in mix mode.

    :return: letters as list
    """
    letters = fancy1_dic.copy()
    fonts = list(set(NON_ASCII_FONTS) - set(MIX_FILTERED_FONTS))
    for i in letters:
        random_font = random.choice(fonts)
        letters[i] = get_font_dic(random_font)[i]
    return letters


def __word2art(word, font, chr_ignore, letters, next_word, sep="\n"):
    """
    Return art word.

    :param word: input word
    :type word: str
    :param font: input font
    :type font: str
    :param chr_ignore: ignore not supported character
    :type chr_ignore: bool
    :param letters: font letters table
    :type letters: dict
    :param next_word: next word flag
    :type next_word: bool
    :param sep: line separator char
    :type sep: str
    :return: ascii art as str
    """
    split_list = []
    result_list = []
    splitter = "\n"
    if isinstance(sep, str):
        splitter = sep
    if not word and next_word:
        return splitter
    for i in word:
        if ord(i) == 9:
            continue
        if i not in letters:
            if (chr_ignore):
                continue
            else:
                raise artError(str(i) + " is invalid.")
        if letters[i]:
            split_list.append(letters[i].split("\n"))
    if font in ["mirror", "mirror_flip"]:
        split_list.reverse()
    if not split_list:
        return ""
    for i in range(len(split_list[0])):
        temp = ""
        for item in split_list:
            temp = temp + item[i]
        result_list.append(temp)
    result = (splitter).join(result_list)
    if result[-1] != "\n" and next_word:
        result += splitter
    return result


def text2art(
        text,
        font=DEFAULT_FONT,
        chr_ignore=True,
        decoration=None,
        sep="\n",
        space=0,
        __detailed_return=False):
    r"""
    Return art text (support \n).

    :param text: input text
    :type text:str
    :param font: input font
    :type font:str
    :param chr_ignore: ignore not supported character
    :type chr_ignore:bool
    :param decoration: text decoration
    :type decoration:str
    :param sep: line separator char
    :type sep: str
    :param space: space between characters
    :type space: int
    :param __detailed_return: flag for returning the font and the decoration
    :type __detailed_return: bool
    :return: ascii art text as str
    """
    letters = standard_dic
    if not isinstance(text, str):
        raise artError(TEXT_TYPE_ERROR)
    if not isinstance(font, str):
        raise artError(FONT_TYPE_ERROR)
    text = (' ' * space).join(text)
    text_temp = text
    font = font.lower()
    if font != "mix":
        font = indirect_font(font, text)
        letters = get_font_dic(font)
        if FONT_MAP[font][1]:
            text_temp = text.lower()
        if font in UPPERCASE_FONTS:
            text_temp = text.upper()
    else:
        letters = mix_letters()
    word_list = text_temp.split("\n")
    result = ""
    next_word_flag = True
    for index, word in enumerate(word_list):
        if index == len(word_list) - 1:
            next_word_flag = False
        result = result + __word2art(word=word,
                                     font=font,
                                     chr_ignore=chr_ignore,
                                     letters=letters,
                                     next_word=next_word_flag,
                                     sep=sep)
    if decoration is not None:
        [decor1, decor2] = decor(decoration, both=True)
        result = decor1 + result + decor2
    if __detailed_return:
        return (result, font, decoration)
    return result


def set_default(
        font=DEFAULT_FONT,
        chr_ignore=True,
        filename="art",
        print_status=True,
        overwrite=False,
        decoration=None,
        sep="\n",
        space=0,
        __detailed_return=False):
    """
    Change text2art, tprint and tsave default values.

    :param font: input font
    :type font:str
    :param chr_ignore: ignore not supported character
    :type chr_ignore:bool
    :param filename: output file name (only tsave)
    :type filename:str
    :param print_status : save message print flag (only tsave)
    :type print_status:bool
    :param overwrite : overwrite the saved file if true (only tsave)
    :type overwrite:bool
    :param decoration: input decoration
    :type decoration:str
    :param sep: line separator char
    :type sep: str
    :param space: space between characters
    :type space: int
    :param __detailed_return: flag for returning the font and the decoration
    :type __detailed_return: bool
    :return: None
    """
    if not isinstance(font, str):
        raise artError(FONT_TYPE_ERROR)
    if not isinstance(decoration, str) and decoration is not None:
        raise artError(DECORATION_TYPE_ERROR)
    if not isinstance(chr_ignore, bool):
        raise artError(CHR_IGNORE_TYPE_ERROR)
    if not isinstance(filename, str):
        raise artError(FILE_TYPE_ERROR)
    if not isinstance(print_status, bool):
        raise artError(PRINT_STATUS_TYPE_ERROR)
    if not isinstance(overwrite, bool):
        raise artError(OVERWRITE_TYPE_ERROR)
    if not isinstance(sep, str):
        raise artError(SEP_TYPE_ERROR)
    if not isinstance(space, int):
        raise artError(SPACE_TYPE_ERROR)
    if not isinstance(__detailed_return, bool):
        raise artError(DETAILED_RETURN_TYPE_ERROR)
    tprint.__defaults__ = (font, chr_ignore, decoration, sep, space)
    tsave.__defaults__ = (
        font,
        filename,
        chr_ignore,
        print_status,
        overwrite,
        decoration,
        sep,
        space)
    text2art.__defaults__ = (
        font,
        chr_ignore,
        decoration,
        sep,
        space,
        __detailed_return)


def get_font_dic(font_name):
    """
    Return given font's dictionary.

    :param  font_name: font's name
    :type font_name:str
    :return: font's dictionary
    """
    return FONT_MAP[font_name][0]


def decor(decoration, reverse=False, both=False):
    """
    Return given decoration part.

    :param  decoration: decoration's name
    :type decoration:str
    :param reverse: true if second tail of decoration wanted
    :type reverse:bool
    :param both: both tails returning flag
    :type bool: bool
    :return: decor's tail as str or tails as list of str
    """
    if not isinstance(decoration, str):
        raise artError(DECORATION_TYPE_ERROR)
    decoration = decoration.lower()
    decoration = indirect_decoration(decoration)
    if both:
        return DECORATIONS_MAP[decoration]
    if reverse:
        return DECORATIONS_MAP[decoration][-1]
    return DECORATIONS_MAP[decoration][0]



# -*- coding: utf-8 -*-
"""Fonts height, duplication and UTF-8 compatibility testing script."""
import itertools as it
import sys
import art
from art.art_param import NON_ASCII_FONTS
from font_wizard import is_utf8, is_ascii

Failed1 = 0
Failed2 = 0
Failed3 = 0
Failed4 = 0
Font_List = list(art.art_param.FONT_MAP)
Message1 = "Font height test "
Message2 = "Font duplication test "
Message3 = "Font UTF-8 compatibility test "
Message4 = "{0}-font duplication -- > {1},{2}"
Message5 = "Font width test "


def print_result(flag_list, message_list):
    """
    Print final result.

    :param flag_list: list of test flags
    :type flag_list: list
    :param message_list: list of test messages
    :type message_list: list
    :return: None
    """
    print("Art version : {}".format(art.__version__))
    print("Number of fonts : {}".format(art.FONT_COUNTER))
    for index, flag in enumerate(flag_list):
        if flag == 0:
            print(message_list[index] + "passed!")
        else:
            print(message_list[index] + "failed!")


if __name__ == "__main__":
    for font in Font_List:
        height_list = []
        font_data = ""
        first_line_list = [x.split("\n")[0] in ["", " "] for x in art.get_font_dic(font).values()]
        last_line_list = [x.split("\n")[-1] in ["", " "] for x in art.get_font_dic(font).values()]
        for letter, letter_data in art.get_font_dic(font).items():
            letter_data_split = letter_data.split("\n")
            width_list = [len(x) for x in letter_data_split]
            if letter_data_split[-1] in ["", " "] and all(last_line_list):
                width_list = width_list[:-1]
            if width_list and letter_data_split[0] in ["", " "] and all(first_line_list):
                width_list = width_list[1:]
            if len(set(width_list)) > 1:
                print(
                    "Width error in font {0}, letter {1}".format(
                        font, letter))
                Failed4 += 1
            if letter_data:
                height_list.append(len(letter_data_split))
            font_data += letter_data
        ascii_flag = is_ascii(font_data)
        if len(set(height_list)) != 1:
            print("Height error in font : " + font)
            Failed1 += 1
        if not is_utf8(font_data):
            Failed3 += 1
            print("UTF-8 compatibility error in font : " + font)
        if ascii_flag and font in NON_ASCII_FONTS:
            print(
                "Font type warning : {0} is ASCII but imported as NON-ASCII".format(font))
        if not ascii_flag and font not in NON_ASCII_FONTS:
            print(
                "Font type warning : {0} is NON-ASCII but imported as ASCII".format(font))

    for font1, font2 in it.combinations(Font_List, 2):
        if len(art.get_font_dic(font1)) == len(art.get_font_dic(font2)):
            if art.get_font_dic(font1) == art.get_font_dic(font2):
                Failed2 += 1
                print(Message4.format(str(Failed2), font1, font2))
        else:
            font1_keys = set(art.get_font_dic(font1))
            font2_keys = set(art.get_font_dic(font2))
            inter_keys = list(font1_keys.intersection(font2_keys))
            font1_map = []
            font2_map = []
            for letter in inter_keys:
                font1_map.append(art.get_font_dic(font1)[letter])
                font2_map.append(art.get_font_dic(font2)[letter])
            if font1_map == font2_map:
                Failed2 += 1
                print(Message4.format(str(Failed2), font1, font2))
    print_result([Failed1, Failed2, Failed3, Failed4], [
                 Message1, Message2, Message3, Message5])
    sys.exit(Failed2 + Failed1 + Failed3 + Failed4)



# -*- coding: utf-8 -*-
"""Setup module."""
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


MINIMAL_DESCRIPTION = '''ASCII art is also known as "computer text art".
    It involves the smart placement of typed special characters or
    letters to make a visual shape that is spread over multiple lines of text.
    ART is a Python lib for text converting to ASCII art fancy.'''


def get_dev_requires():
    """Read dev-requirements.txt."""
    requirements = open("dev-requirements.txt", "r").read()
    return [x for x in requirements.split() if x]


def read_description():
    """Read README.md and CHANGELOG.md."""
    try:
        with open("README.md") as r:
            description = "\n"
            description += r.read()
        with open("CHANGELOG.md") as c:
            description += "\n"
            description += c.read()
        return description
    except Exception:
        return MINIMAL_DESCRIPTION


setup(
    name='art',
    packages=['art'],
    version='6.1',
    description='ASCII Art Library For Python',
    long_description=read_description(),
    long_description_content_type='text/markdown',
    author='Sepand Haghighi',
    author_email='info@ascii-art.site',
    url='https://github.com/sepandhaghighi/art',
    keywords="ascii art python3 python text font non-ascii printing",
    project_urls={
        'Webpage': 'https://www.ascii-art.site',
        'Source': 'https://github.com/sepandhaghighi/art',
        'Tracker': 'https://github.com/sepandhaghighi/art/issues',
        'Discord': 'https://discord.com/invite/FAAyq3QJqP',
    },
    install_requires=[],
    extras_require={
        "dev": get_dev_requires()
    },
    python_requires='>=3.5',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Science/Research',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Topic :: Text Processing :: Fonts',
        'Topic :: Text Editors',
        'Topic :: Text Processing :: General',
        'Topic :: Utilities',
        'Topic :: Multimedia',
        'Topic :: Printing',
    ],
    license='MIT',
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'art = art.__main__:main',
        ]}
)



# -*- coding: utf-8 -*-
"""Art parameters."""
from .text_dic1 import *
from .text_dic2 import *
from .text_dic3 import *
from .decor_dic import *
from .art_dic import *

ART_VERSION = "6.1"  # pragma: no cover
FONT_SMALL_THRESHOLD = 50  # pragma: no cover
FONT_MEDIUM_THRESHOLD = 100  # pragma: no cover
FONT_LARGE_THRESHOLD = 200  # pragma: no cover

TEXT_XLARGE_THRESHOLD = 3  # pragma: no cover
TEXT_LARGE_THRESHOLD = 7  # pragma: no cover
TEXT_MEDIUM_THRESHOLD = 10  # pragma: no cover

ART_TYPE_ERROR = "The 'artname' type must be str."
ART_NAME_ERROR = "Invalid art name."
NUMBER_TYPE_ERROR = "The 'number' type must be int."
TEXT_TYPE_ERROR = "The 'text' type must be str."
FONT_TYPE_ERROR = "The 'font' type must be str."
DECORATION_TYPE_ERROR = "The 'decoration' type must be str."
CHR_IGNORE_TYPE_ERROR = "The 'chr_ignore' type must be bool."
FILE_TYPE_ERROR = "The 'filename' type must be str."
PRINT_STATUS_TYPE_ERROR = "The 'print_status' type must be bool."
OVERWRITE_TYPE_ERROR = "The 'overwrite' type must be bool."
SEP_TYPE_ERROR = "The 'sep' type must be str."
SPACE_TYPE_ERROR = "The 'space' type must be int."
DETAILED_RETURN_TYPE_ERROR = "The '__detailed_return' type must be bool."
ART_ENVIRONMENT_WARNING = "[Warning] '{0}' art is not printable in this environment."
FONT_ENVIRONMENT_WARNING = "[Warning] '{0}' font is not printable in this environment."
FONT_OR_DECOR_ENVIRONMENT_WARNING = "[Warning] '{0}' font or '{1}' decoration is not printable in this environment."
PACKAGE_LOAD_WARNING = "[Warning] There is a problem loading the package 'coverage'."

CLI_HELP = """
Webpage : https://www.ascii-art.site

Help :

\t- list --> (list of arts)

\t- fonts --> (list of fonts)

\t- test --> (run tests)

\t- text [yourtext] [font(optional)] --> (text art) Example : 'art text exampletext block'

\t- shape [shapename] --> (shape art) Example : 'art shape butterfly'

\t- save [yourtext] [font(optional)]  -->  Example : 'art save exampletext block'

\t- all [yourtext]  -->  Example : 'art all exampletext'
"""

SMALL_WIZARD_FONT = [
    "contessa",
    "avatar",
    "mini",
    "bigfig",
    "lcd",
    "bubble",
    "digital",
    "twopoint",
    "3x5",
    "threepoint",
    "ascii_new_roman",
    "bulbhead",
    "lockergnome"]  # pragma: no cover
MEDIUM_WIZARD_FONT = [
    "soft",
    "4max",
    "5x7",
    "stampate",
    "o8",
    "standard",
    "alphabet",
    "computer",
    "shadow",
    "speed",
    "rounded",
    "chartri"]  # pragma: no cover
LARGE_WIZARD_FONT = [
    "xhelvi",
    "utopiai",
    "rev",
    "smpoison",
    "3-d",
    "larry3d",
    "nancyj",
    "nancyj-underlined",
    "os2",
    "block2"]  # pragma: no cover
XLARGE_WIZARD_FONT = [
    "dotmatrix",
    "univers",
    "doh",
    "roman",
    "hollywood",
    "georgia11",
    "block"]  # pragma: no cover

UPPERCASE_FONTS = [
    "1943",
    "4x4_offr",
    "64f1",
    "a_zooloo",
    "advenger",
    "aquaplan",
    "assalt_m",
    "asslt_m",
    "atc",
    "atc_gran",
    "battlesh",
    "baz_bil",
    "beer_pub",
    "c1",
    "c2",
    "c_consen",
    "char1",
    "char4",
    "coil_cop",
    "druid",
    "faces_of",
    "fair_mea",
    "fairligh",
    "fantasy1",
    "fbr1",
    "fbr12",
    "fbr_stri",
    "fbr_tilt",
    "finalass",
    "fp1",
    "fp2",
    "funky_dr",
    "future_1",
    "future_2",
    "future_3",
    "future_4",
    "future_5",
    "future_6",
    "future_7",
    "future_8",
    "ghost_bo",
    "grand_pr",
    "green_be",
    "hades",
    "heavy_me",
    "heroboti",
    "house_of",
    "hypa_bal",
    "hyper",
    "kgames_i",
    "kik_star",
    "krak_out",
    "tsn_base",
    "ugalympi",
    "unarmed",
    "usa",
    "usa_pq",
    "vortron",
    "war_of_w",
    "yie-ar",
    "yie_ar_k",
    "z-pilot",
    "zig_zag",
    "zone7"]

NON_ASCII_ARTS = [
    '3',
    '5',
    '9/11 truth',
    'acid',
    'afraid',
    'airplane1',
    'airplane2',
    'airplane3',
    'ak-47',
    'almost cared',
    'american money2',
    'american money3',
    'american money4',
    'american money5',
    'angry birds',
    'angry face',
    'angry face2',
    'angry troll',
    'angry',
    'angry2',
    'ankush',
    'arrow1',
    'arrowhead',
    'at what cost',
    'awkward',
    'band aid',
    'barbell',
    'barcode1',
    'barcode2',
    'barf',
    'basking in glory',
    'bautista',
    'bear GTFO',
    'bear squiting',
    'bear',
    'bear2',
    'because',
    'bee',
    'being draged',
    'bender',
    'big eyes',
    'big nose',
    'bird',
    'birds',
    'bomb',
    'boombox1',
    'boombox2',
    'boxing',
    'breakdown',
    'british money',
    'bullshit',
    'butt',
    'butterfly',
    'camera',
    'car race',
    'careless',
    'cassette1',
    'cassette2',
    'cat face',
    'cat smile',
    'cat2',
    'cat4',
    'catlenny',
    'chair',
    'chasing',
    'cheer',
    'cheers',
    'chess pieces',
    'chess',
    'chicken',
    'chu',
    'cigarette1',
    'club bold',
    'club regular',
    'coffee now',
    'confused scratch',
    'confused3',
    'confused5',
    'confused6',
    'confused7',
    'confused8',
    'confused9',
    'crayons',
    'crazy',
    'creeper',
    'cry face',
    'cry troll',
    'cry',
    'crying1',
    'cup1',
    'cup2',
    'cute cat',
    'cute face',
    'cute face2',
    'cute face3',
    'cute face4',
    'cute face5',
    'cute face6',
    'cute face7',
    'cute face8',
    'cute face9',
    'upset',
    'dab',
    'dalek',
    'damnyou',
    'dance2',
    'dancee',
    'dancing people',
    'dancing',
    'dead eyes',
    'dear god why',
    'decorate',
    'depressed',
    'derp',
    'devious smile',
    'dgaf',
    'diamond bold',
    'diamond regular',
    'disagree',
    'discombobulated',
    'dislike1',
    'dislike2',
    'do you even lift bro?',
    'double flip',
    'drowning',
    'drunkenness',
    'dunno',
    'dunno2',
    'dunno3',
    'dunno4',
    'dunno5',
    'electrocardiogram1',
    'electrocardiogram2',
    'electrocardiogram3',
    'electrocardiogram4',
    'elephant',
    'emo dance',
    'energy',
    'envelope',
    'epic gun',
    'equalizer',
    'error',
    'exchange',
    'excited',
    'exorcism',
    'eye closed',
    'eye roll',
    'eyes',
    'face',
    'facepalm',
    'fail',
    'fart',
    'feel perky',
    'fido',
    'fight',
    'finger1',
    'finger2',
    'finger3',
    'finger4',
    'finn',
    'fish invasion',
    'fish swim',
    'fish3',
    'fisticuffs',
    'flex',
    'flip friend',
    'fly away',
    'flying',
    'formula1 car',
    'french kiss',
    'frown',
    'fu',
    'fuck you2',
    'fuckall',
    'fungry',
    'ghost',
    'gimme',
    'glasses2',
    'glitter',
    'go away bear',
    'gotit',
    'gtalk fit',
    'guitar',
    'gun1',
    'gun2',
    'gun3',
    'gun4',
    'hairstyle',
    'happy birthday 1',
    'happy face',
    'happy hug',
    'happy square',
    'happy',
    'happy2',
    'happy3',
    'happy6',
    'happy7',
    'happy8',
    'happy9',
    'happy10',
    'happy11',
    'happy12',
    'happy13',
    'happy14',
    'happy15',
    'happy16',
    'happy17',
    'happy18',
    'happy19',
    'happy20',
    'happy21',
    'happy22',
    'happy23',
    'happy24',
    'happy25',
    'happy26',
    'happy27',
    'head shot',
    'heart bold',
    'heart regular',
    'heart1',
    'heart2',
    'hell yeah',
    'hello',
    'hello2',
    'help',
    'high five',
    'hitchhicking',
    'honeycute',
    'house',
    'hug me',
    'hugger',
    'huhu',
    'hybrix',
    'i dont care',
    'i kill you',
    'im a hugger',
    'injured',
    'inlove',
    'innocent face',
    'japanese lion face',
    'jaymz',
    'jazz musician',
    'john lennon',
    'joker1',
    'joker2',
    'joker3',
    'joker4',
    'jokeranonimous',
    'jokeranonimous2',
    'judgemental',
    'judging',
    'kablewee',
    'killer',
    'kilroy was here',
    'kirby',
    'kissing',
    'kissing2',
    'kissing3',
    'kissing4',
    'kissing5',
    'kissing6',
    'kiss2',
    'kiss3',
    'kitty emote',
    'kokain',
    'kyubey',
    'kyubey2',
    'laughing',
    'lenny',
    'line brack',
    'listening to headphones',
    'loading1',
    'loading2',
    'loading3',
    'loading4',
    'loading5',
    'loading6',
    'looking face',
    'love in my eye1',
    'love in my eye2',
    'love in my eye3',
    'love',
    'love2',
    'love3',
    'love4',
    'man spider',
    'spider4',
    'man tears',
    'mango',
    'med man',
    'med',
    'meditation',
    'meep',
    'meow',
    'monocle',
    'monster',
    'monster2',
    'mouse2',
    'mouse3',
    'mouse4',
    'myancat',
    'nathan',
    'needle1',
    'neo',
    'no support',
    'nose',
    'opera',
    'owlkin',
    'pac man',
    'panda',
    'party time',
    'peace yo!',
    'peepers',
    'penis2',
    'perky',
    'pig2',
    'piggy',
    'ping pong',
    'pirate',
    'pistols1',
    'pistols2',
    'pistols3',
    'pistols4',
    'pistols5',
    'mad',
    'mad2',
    'mad5',
    'mad6',
    'mad7',
    'mad9',
    'mad10',
    'playing cards clubs',
    'playing cards clubs waterfall',
    'playing cards diamonds',
    'playing cards diamonds waterfall',
    'playing cards hearts',
    'playing cards hearts waterfall',
    'playing cards spades',
    'playing cards spades waterfall',
    'playing cards',
    'playing cards waterfall',
    'playing cards waterfall (trump)',
    'playing in snow',
    'point',
    'polar bear',
    'pretty eyes',
    'professor',
    'puls',
    'put the table back',
    'rak',
    'rare',
    'real face',
    'religious',
    'resting my eyes',
    'robber',
    'robot boy',
    'robot2',
    'robot3',
    'rocket',
    'rope',
    'round bird',
    'running',
    'sad and confused',
    'sad and crying',
    'sad face',
    'sad1',
    'sad2',
    'sad3',
    'sad4',
    'sad5',
    'sat',
    'satan',
    'satisfied',
    'scissors',
    'seal',
    'sex symbol',
    'shark face',
    'sheep',
    'shocked1',
    'shrug',
    'shy',
    'singing',
    'singing2',
    'sky free',
    'sleepy coffee',
    'sleepy',
    'slenderman',
    'smooth',
    'smug bastard',
    'sniper rifle',
    'sniperstars',
    'snowing',
    'snowman1',
    'sorreh bro',
    'spade bold',
    'spade regular',
    'sparkling heart',
    'spell cast',
    'squee',
    'squid',
    'srs face',
    'staring',
    'stars',
    'stars2',
    'stranger danger',
    'strut',
    'stunna shades',
    'sunglasses1',
    'sunny day',
    'surprised2',
    "surprised4",
    "surprised5",
    "surprised6",
    "surprised8",
    "surprised9",
    "surprised10",
    "surprised11",
    "surprised12",
    "surprised13",
    "surprised15",
    "surprised16",
    "surprised17",
    "surprised18",
    "surprised19",
    "surprised20",
    'sword2',
    'sword3',
    'sword8',
    'sword9',
    'table flip',
    'table flip2',
    'table flip3',
    'table flip4',
    'table flip5',
    'table flip6',
    'table flip7',
    'table flip8',
    'table flip9',
    'table flip10',
    'taking a dump',
    'teddy',
    'telephone',
    'tgif',
    'things that can_t be unseen',
    'tidy up',
    'tie-fighter',
    'tired',
    'touchy feely',
    'toungue out1',
    'train',
    'tripping out',
    'trolling',
    'umadbro',
    'up',
    'upsidedown',
    'wat',
    'wat-wat',
    'waves',
    'weather',
    'what?',
    'what??',
    'whisling',
    'why',
    'winnie the pooh',
    'winning',
    'wizard',
    'wizard2',
    'woman',
    'worried',
    'wtf dude?',
    'yessir',
    'yolo',
    'yun',
    'zable',
    'zoidberg',
    'zombie',
    'zombie2',
    'zoned',
    'swim',
    'swim2',
    'swim3',
    'yawning',
    'boobs2']

RANDOM_FILTERED_ARTS = [] + NON_ASCII_ARTS


NON_ASCII_FONTS = [
    "contouring1",
    "contouring2",
    "contouring3",
    "contouring4",
    "mirror",
    "mirror_flip",
    "flip",
    "white_bubble",
    "smallcaps2",
    "superscript",
    "subscript1",
    "full_width",
    "antrophobia",
    "currency",
    "special",
    "dirty",
    "knight",
    "thin2",
    "tiny",
    "love1",
    "love2",
    "fancy1",
    "fancy2",
    "fancy3",
    "fancy4",
    "fancy5",
    "fancy6",
    "fancy7",
    "fancy8",
    "fancy9",
    "fancy10",
    "fancy11",
    "fancy12",
    "fancy13",
    "fancy14",
    "fancy15",
    "fancy16",
    "fancy17",
    "fancy18",
    "fancy19",
    "fancy20",
    "fancy21",
    "fancy22",
    "fancy23",
    "fancy24",
    "fancy25",
    "fancy26",
    "fancy27",
    "fancy28",
    "fancy29",
    "fancy30",
    "symbols",
    "fancy31",
    "fancy32",
    "fancy33",
    "fancy34",
    "fancy35",
    "fancy36",
    "fancy37",
    "fancy38",
    "fancy39",
    "fancy40",
    "fancy41",
    "fancy42",
    "fancy43",
    "fancy44",
    "fancy45",
    "fancy46",
    "fancy47",
    "fancy48",
    "fancy49",
    "fancy50",
    "fancy52",
    "fancy53",
    "fancy54",
    "fancy55",
    "fancy56",
    "fancy57",
    "fancy58",
    "fancy59",
    "fancy60",
    "fancy61",
    "fancy62",
    "fancy63",
    "fancy64",
    "fancy65",
    "fancy66",
    "fancy67",
    "fancy68",
    "fancy69",
    "fancy70",
    "fancy71",
    "fancy72",
    "fancy73",
    "fancy74",
    "fancy75",
    "fancy76",
    "fancy77",
    "fancy78",
    "fancy79",
    "fancy80",
    "fancy81",
    "fancy82",
    "fancy83",
    "fancy84",
    "fancy85",
    "fancy86",
    "fancy87",
    "fancy88",
    "fancy89",
    "fancy90",
    "fancy91",
    "fancy92",
    "fancy93",
    "fancy94",
    "fancy95",
    "fancy96",
    "fancy97",
    "fancy98",
    "fancy99",
    "fancy100",
    "fancy101",
    "fancy102",
    "fancy103",
    "fancy104",
    "fancy105",
    "fancy106",
    "fancy107",
    "fancy108",
    "fancy109",
    "fancy110",
    "fancy111",
    "fancy112",
    "fancy113",
    "fancy114",
    "fancy115",
    "fancy116",
    "fancy117",
    "fancy118",
    "fancy119",
    "fancy120",
    "fancy121",
    "fancy122",
    "fancy123",
    "fancy124",
    "fancy125",
    "fancy126",
    "fancy127",
    "fancy128",
    "fancy129",
    "fancy131",
    "fancy132",
    "fancy133",
    "fancy134",
    "fancy135",
    "fancy136",
    "fancy137",
    "fancy138",
    "fancy139",
    "fancy140",
    "fancy141",
    "fancy142",
    "fancy143",
    "fancy144",
    "fancy145",
    "fancy146",
    "fancy147",
    "fancy148",
    "foxy",
    "white_square",
    "black_square",
    "paranormal",
    "black_bubble",
    "smallcaps3",
    "fraktur2",
    "bold_fraktur",
    "bold_script",
    "handwriting1",
    "handwriting2",
    "rusify",
    "monospace",
    "dirty2",
    "knight2",
    "thin3",
    "tiny2",
    "upside_down1",
    "chinese_mafia",
    "sunday_cuddle",
    "native_lands",
    "foreign_friends",
    "atlantic",
    "funky_fresh",
    "greek_legends",
    "minion",
    "hippie",
    "sarah",
    "dwarf",
    "lilia",
    "smooth1",
    "smooth2",
    "carrier1",
    "carrier2",
    "awesome",
    "cjk",
    "cute1",
    "cute2",
    "cute3",
    "malayalam",
    "cranky",
    "lopioo",
    "scammer",
    "strikethrough",
    "drako",
    "celtic",
    "fasion",
    "curly",
    "russian",
    "russian2",
    "messletters",
    "coptic1",
    "swirly",
    "dotted",
    "coptic2",
    "parenthesized",
    "strange",
    "wiggly",
    "hazy",
    "hyves",
    "awcute",
    "slammer",
    "sheqi",
    "fari",
    "shasha",
    "pin1",
    "pin2",
    "sign",
    "bud1",
    "bud2",
    "old_italic",
    "ninja",
    "tai_viet",
    "subscript2",
    "fancy130",
    "smooth3",
    "flaky",
    "tiny_caps",
    "high_above",
    "handwriting3",
    "fantasy2",
    "manga",
    "upside_down2",
    "cool",
    "instagram",
    "tarty1",
    "tarty2",
    "tarty3",
    "tarty4",
    "tarty5",
    "tarty6",
    "tarty7",
    "tarty8",
    "tarty9",
    "got",
    "vip",
    "crazy1",
    "crazy2",
    "crazy3",
    "cruse",
    "drippy",
    "stylish",
    "ayda",
    "small_fancy",
    "boom",
    "cross_jesus",
    "dark_tattoo",
    "lucifer",
    "ascii_roman",
    "squiggle1",
    "squiggle2",
    "vaporwave",
    "emoji"
]

RANDOM_FILTERED_FONTS = [
    '5x8',
    'binary',
    'decimal',
    'high_noo',
    'hills',
    'katakana',
    'morse',
    'nfi1',
    'octal',
    'rot13',
    'smtengwar',
    'tengwar',
    'tsalagi',
    'gauntlet',
    'flyn_sh',
    'moscow',
    'dwhistled'] + NON_ASCII_FONTS

MIX_FILTERED_FONTS = [
    "tarty1",
    "tarty2",
    "tarty3",
    "tarty4",
    "tarty5",
    "tarty6",
    "tarty7",
    "tarty8",
    "tarty9"]

DESCRIPTION = '''ASCII art is also known as "computer text art".
It involves the smart placement of typed special characters or
letters to make a visual shape that is spread over multiple lines of text.
ART is a Python lib for text converting to ASCII art fancy.'''  # pragma: no cover


FONT_MAP = {"block": [block_dic, True], "banner": [banner_dic, False],  # pragma: no cover
            "standard": [standard_dic, False], "avatar": [avatar_dic, True],
            "basic": [basic_dic, True], "bulbhead": [bulbhead_dic, True],
            "chunky": [chunky_dic, False], "coinstak": [coinstak_dic, False],
            "contessa": [contessa_dic, False], "contrast": [contrast_dic, True],
            "cyberlarge": [cyberlarge_dic, True], "cybermedium": [cybermedium_dic, True],
            "doom": [doom_dic, False], "dotmatrix": [dotmatrix_dic, False],
            "drpepper": [drpepper_dic, False],
            "epic": [epic_dic, True], "fuzzy": [fuzzy_dic, False],
            "isometric1": [isometric1_dic, True], "isometric2": [isometric2_dic, True],
            "isometric3": [isometric3_dic, True], "isometric4": [isometric4_dic, True],
            "larry3d": [larry3d_dic, False],
            "nancyj": [nancyj_dic, False], "ogre": [ogre_dic, False],
            "rectangles": [rectangles_dic, False], "roman": [roman_dic, False],
            "rounded": [rounded_dic, False], "rowancap": [rowancap_dic, True],
            "script": [script_dic, False],
            "serifcap": [serifcap_dic, True], "shadow": [shadow_dic, False],
            "slant": [slant_dic, False], "speed": [speed_dic, False],
            "starwars": [starwars_dic, False], "stop": [stop_dic, False],
            "thin": [thin_dic, False], "usaflag": [usaflag_dic, False],
            "3-d": [dic_3d, False], "3x5": [dic_3x5, False], "5lineoblique":
                [dic_5lineoblique, False], "alphabet": [alphabet_dic, False],
            "banner3-d": [banner3d_dic, True],
            "banner3": [banner3_dic, True], "banner4": [banner4_dic, True],
            "bell": [bell_dic, False], "catwalk": [catwalk_dic, False], "colossal": [colossal_dic, False],
            "acrobatic": [acrobatic_dic, True], "alligator": [alligator_dic, False],
            "alligator2": [alligator2_dic, False], "block2": [block2_dic, True],
            "caligraphy": [caligraphy_dic, True],
            "computer": [computer_dic, True], "digital": [digital_dic, True],
            "doh": [doh_dic, True],
            "eftirobot": [eftirobot_dic, True], "graffiti": [graffiti_dic, True],
            "stellar": [stellar_dic, False], "swan": [swan_dic, False], "tanja": [tanja_dic, False],
            "thick": [thick_dic, False], "threepoint": [threepoint_dic, False],
            "tombstone": [tombstone_dic, True], "trek": [trek_dic, True],
            "twopoint": [twopoint_dic, False], "univers": [univers_dic, False],
            "weird": [weird_dic, False], "pebbles": [pebbles_dic, False],
            "puffy": [puffy_dic, False], "tinker-toy": [tinker_toy_dic, False],
            "straight": [straight_dic, False], "stampatello": [stampatello_dic,
                                                               False],
            "smslant": [smslant_dic, False], "smshadow": [smshadow_dic, False],
            "smscript": [smscript_dic, False], "smkeyboard": [smkeyboard_dic,
                                                              False],
            "smisome1": [smisome1_dic, True], "slscript": [slscript_dic, False],
            "slide": [slide_dic, False], "sblood": [sblood_dic, True],
            "rozzo": [rozzo_dic, False], "pyramid": [pyramid_dic, False],
            "maxfour": [maxfour_dic, False], "nipples": [nipples_dic, False],
            "o8": [o8_dic, False], "peaks": [peaks_dic, False],
            "pawp": [pawp_dic, False],
            "barbwire": [barbwire_dic, False], "bigchief": [bigchief_dic, False],
            "binary": [binary_dic, False], "bubble": [bubble_dic, False],
            "calgphy2": [calgphy2_dic, False],
            "cygnet": [cygnet_dic, False], "diamond": [diamond_dic, False],
            "eftifont": [eftifont_dic, False], "eftitalic": [eftitalic_dic,
                                                             False],
            "eftiwater": [eftiwater_dic, False], "fourtops": [fourtops_dic, False],
            "goofy": [goofy_dic, True], "hollywood": [hollywood_dic, False],
            "invita": [invita_dic, False], "italic": [italic_dic, False],
            "jazmine": [jazmine_dic, False], "lcd": [lcd_dic, False],
            "lean": [lean_dic, False], "letters": [letters_dic, False],
            "lockergnome": [lockergnome_dic, False], "madrid": [madrid_dic, False],
            "marquee": [marquee_dic, False], "mike": [mike_dic, True],
            "mini": [mini_dic, False],
            "nancyj-fancy": [nancyj_fancy_dic, False],
            "nancyj-underlined": [nancyj_underlined_dic, False],
            "pepper": [pepper_dic, False], "poison": [poison_dic, True],
            "rot13": [rot13_dic, False], "short": [short_dic, False],
            "small": [small_dic, False], "tengwar": [tengwar_dic, True],
            "big": [big_dic, False], "1row": [dic_1row, True],
            "3d_diagonal": [dic_3d_diagonal, False],
            "4max": [dic_4max, True],
            "amc3line": [amc3line_dic, True],
            "cybersmall": [cybersmall_dic, True],
            "gothic": [gothic_dic, False],
            "rev": [rev_dic, False],
            "smtengwar": [smtengwar_dic, False],
            "term": [term_dic, False],
            "amcrazor": [amcrazor_dic, True],
            "amcaaa01": [amcaaa01_dic, True],
            "amcneko": [amcneko_dic, False],
            "amcrazo2": [amcrazo2_dic, True],
            "amcslash": [amcslash_dic, False],
            "amcthin": [amcthin_dic, True],
            "amctubes": [amctubes_dic, True],
            "amcun1": [amcun1_dic, False],
            "arrows": [arrows_dic, False],
            "bear": [bear_dic, True],
            "benjamin": [benjamin_dic, True],
            "bigfig": [bigfig_dic, False],
            "bolger": [bolger_dic, False],
            "braced": [braced_dic, True],
            "bright": [bright_dic, True],
            "broadway": [broadway_dic, True],
            "cards": [cards_dic, True],
            "chiseled": [chiseled_dic, True],
            "cola": [cola_dic, False],
            "crawford": [crawford_dic, True],
            "cricket": [cricket_dic, False],
            "danc4": [danc4_dic, False],
            "dancingfont": [dancingfont_dic, True],
            "decimal": [decimal_dic, False],
            "defleppard": [defleppard_dic, True],
            "dietcola": [dietcola_dic, False],
            "double": [double_dic, True],
            "doubleshorts": [doubleshorts_dic, True],
            "eftipiti": [eftipiti_dic, False],
            "filter": [filter_dic, True],
            "flipped": [flipped_dic, True],
            "fraktur": [fraktur_dic, False],
            "funface": [funface_dic, True],
            "funfaces": [funfaces_dic, True],
            "georgi16": [georgi16_dic, False],
            "georgia11": [georgia11_dic, False],
            "ghost": [ghost_dic, True],
            "ghoulish": [ghoulish_dic, True],
            "glenyn": [glenyn_dic, True],
            "graceful": [graceful_dic, True],
            "greek": [greek_dic, False],
            "heartleft": [heartleft_dic, False],
            "heartright": [heartright_dic, False],
            "henry3d": [henry3d_dic, False],
            "horizontalleft": [horizontalleft_dic, True],
            "horizontalright": [horizontalright_dic, True],
            "icl-1900": [ICL_1900_dic, True],
            "impossible": [impossible_dic, True],
            "jacky": [jacky_dic, True],
            "katakana": [katakana_dic, False],
            "keyboard": [keyboard_dic, False],
            "knob": [knob_dic, True],
            "lildevil": [lildevil_dic, True],
            "lineblocks": [lineblocks_dic, True],
            "merlin1": [merlin1_dic, True],
            "merlin2": [merlin2_dic, True],
            "modular": [modular_dic, True],
            "morse": [morse_dic, True],
            "moscow": [moscow_dic, True],
            "muzzle": [muzzle_dic, True],
            "nscript": [nscript_dic, False],
            "nvscript": [nvscript_dic, False],
            "octal": [octal_dic, False],
            "oldbanner": [oldbanner_dic, False],
            "os2": [os2_dic, False],
            "puzzle": [puzzle_dic, True],
            "rammstein": [rammstein_dic, False],
            "red_phoenix": [red_phoenix_dic, False],
            "runyc": [runyc_dic, False],
            "santaclara": [santaclara_dic, False],
            "shimrod": [shimrod_dic, False],
            "smallcaps": [smallcaps_dic, True],
            "smpoison": [smpoison_dic, True],
            "soft": [soft_dic, False],
            "spliff": [spliff_dic, True],
            "stacey": [stacey_dic, True],
            "stampate": [stampate_dic, False],
            "stforek": [stforek_dic, True],
            "sub-zero": [sub_zero_dic, True],
            "swampland": [swampland_dic, True],
            "sweet": [sweet_dic, True],
            "ticks": [ticks_dic, False],
            "ticksslant": [ticksslant_dic, False],
            "tiles": [tiles_dic, False],
            "tsalagi": [tsalagi_dic, False],
            "tubular": [tubular_dic, False],
            "twisted": [twisted_dic, True],
            "varsity": [varsity_dic, False],
            "wavy": [wavy_dic, False],
            "wetletter": [wetletter_dic, True],
            "whimsy": [whimsy_dic, True],
            "wow": [wow_dic, True],
            "alligator3": [alligator3_dic, True],
            "alpha": [alpha_dic, True],
            "amc3liv1": [amc3liv1_dic, True],
            "ascii_new_roman": [ascii_new_roman_dic, True],
            "b1ff": [B1FF_dic, True],
            "dwhistled": [dwhistled_dic, False],
            "eftiwall": [eftiwall_dic, False],
            "fire_font-s": [fire_font_s_dic, False],
            "gradient": [gradient_dic, True],
            "1943": [dic_1943, False],
            "advenger": [advenger_dic, False],
            "char1": [char1_dic, False],
            "char2": [char2_dic, False],
            "char3": [char3_dic, False],
            "char4": [char4_dic, False],
            "charact1": [charact1_dic, False],
            "charact2": [charact2_dic, False],
            "charact3": [charact3_dic, False],
            "charact4": [charact4_dic, False],
            "charact5": [charact5_dic, False],
            "charact6": [charact6_dic, False],
            "characte": [characte_dic, False],
            "chartr": [chartr_dic, False],
            "chartri": [chartri_dic, False],
            "xbrite": [xbrite_dic, False],
            "xbriteb": [xbriteb_dic, False],
            "xbritebi": [xbritebi_dic, False],
            "xbritei": [xbritei_dic, False],
            "xchartr": [xchartr_dic, False],
            "xchartri": [xchartri_dic, False],
            "xcour": [xcour_dic, False],
            "xcourb": [xcourb_dic, False],
            "xcourbi": [xcourbi_dic, False],
            "xcouri": [xcouri_dic, False],
            "xhelv": [xhelv_dic, False],
            "xhelvb": [xhelvb_dic, False],
            "xhelvbi": [xhelvbi_dic, False],
            "xhelvi": [xhelvi_dic, False],
            "xsans": [xsans_dic, False],
            "xsansb": [xsansb_dic, False],
            "xsansbi": [xsansbi_dic, False],
            "xsansi": [xsansi_dic, False],
            "xtimes": [xtimes_dic, False],
            "xttyb": [xttyb_dic, False],
            "heroboti": [heroboti_dic, False],
            "high_noo": [high_noo_dic, False],
            "hills": [hills_dic, False],
            "home_pak": [home_pak_dic, True],
            "house_of": [house_of_dic, False],
            "hypa_bal": [hypa_bal_dic, False],
            "hyper": [hyper_dic, False],
            "inc_raw": [inc_raw_dic, False],
            "italics": [italics_dic, False],
            "kgames_i": [kgames_i_dic, False],
            "4x4_offr": [dic_4x4_offr, False],
            "5x7": [dic_5x7, False],
            "5x8": [dic_5x8, False],
            "6x9": [dic_6x9, False],
            "6x10": [dic_6x10, False],
            "64f1": [dic_64f1, False],
            "a_zooloo": [a_zooloo_dic, False],
            "asc": [asc_dic, False],
            "assalt_m": [assalt_m_dic, False],
            "asslt_m": [asslt__m_dic, False],
            "atc": [atc_dic, False],
            "atc_gran": [atc_gran_dic, False],
            "battlesh": [battlesh_dic, False],
            "baz_bil": [baz_bil_dic, False],
            "beer_pub": [beer_pub_dic, False],
            "c1": [c1_dic, False],
            "c2": [c2_dic, False],
            "kik_star": [kik_star_dic, False],
            "krak_out": [krak_out_dic, False],
            "tsn_base": [tsn_base_dic, False],
            "twin_cob": [twin_cob_dic, False],
            "type_set": [type_set_dic, False],
            "ucf_fan": [ucf_fan_dic, False],
            "ugalympi": [ugalympi_dic, False],
            "unarmed": [unarmed__dic, False],
            "usa": [usa_dic, False],
            "usa_pq": [usa_pq_dic, False],
            "utopia": [utopia_dic, False],
            "utopiab": [utopiab_dic, False],
            "utopiabi": [utopiabi_dic, False],
            "utopiai": [utopiai_dic, False],
            "vortron": [vortron_dic, False],
            "war_of_w": [war_of_w_dic, False],
            "xtty": [xtty_dic, False],
            "love1": [love1_dic, False],
            "yie_ar_k": [yie_ar_k_dic, False],
            "yie-ar": [yie_ar_dic, False],
            "zig_zag": [zig_zag_dic, False],
            "zone7": [zone7_dic, False],
            "z-pilot": [z_pilot_dic, False],
            "aquaplan": [aquaplan_dic, False],
            "ascii": [ascii_dic, False],
            "c_ascii": [c_ascii_dic, False],
            "c_consen": [c_consen_dic, False],
            "clb6x10": [clb6x10_dic, False],
            "clb8x8": [clb8x8_dic, False],
            "clb8x10": [clb8x10_dic, False],
            "cli8x8": [cli8x8_dic, False],
            "clr4x6": [clr4x6_dic, False],
            "clr5x6": [clr5x6_dic, False],
            "clr5x8": [clr5x8_dic, False],
            "clr5x10": [clr5x10_dic, False],
            "clr6x6": [clr6x6_dic, False],
            "clr6x8": [clr6x8_dic, False],
            "clr6x10": [clr6x10_dic, False],
            "clr7x8": [clr7x8_dic, False],
            "clr8x8": [clr8x8_dic, False],
            "clr8x10": [clr8x10_dic, False],
            "coil_cop": [coil_cop_dic, False],
            "contouring1": [contouring1_dic, False],
            "contouring2": [contouring2_dic, True],
            "com_sen": [com_sen_dic, False],
            "druid": [druid_dic, False],
            "e_fist": [e_fist_dic, True],
            "ebbs_1": [ebbs_1_dic, False],
            "ebbs_2": [ebbs_2_dic, False],
            "eca": [eca_dic, False],
            "faces_of": [faces_of_dic, False],
            "fair_mea": [fair_mea_dic, False],
            "fairligh": [fairligh_dic, False],
            "fantasy1": [fantasy1_dic, False],
            "fbr_stri": [fbr_stri_dic, False],
            "fbr_tilt": [fbr_tilt_dic, False],
            "fbr1": [fbr1_dic, False],
            "fbr2": [fbr2_dic, False],
            "fbr12": [fbr12_dic, False],
            "finalass": [finalass_dic, False],
            "fireing": [fireing_dic, False],
            "flyn_sh": [flyn_sh_dic, False],
            "fp1": [fp1_dic, False],
            "fp2": [fp2_dic, False],
            "funky_dr": [funky_dr_dic, False],
            "future_1": [future_1_dic, False],
            "future_2": [future_2_dic, False],
            "future_3": [future_3_dic, False],
            "future_4": [future_4_dic, False],
            "future_5": [future_5_dic, False],
            "future_6": [future_6_dic, False],
            "future_7": [future_7_dic, False],
            "future_8": [future_8_dic, False],
            "gauntlet": [gauntlet_dic, False],
            "ghost_bo": [ghost_bo_dic, False],
            "grand_pr": [grand_pr_dic, False],
            "green_be": [green_be_dic, False],
            "hades": [hades_dic, False],
            "heavy_me": [heavy_me_dic, False],
            "nfi1": [nfi1_dic, False],
            "flip": [flip_dic, False],
            "mirror": [mirror_dic, False],
            "mirror_flip": [mirror_flip_dic, False],
            "white_bubble": [white_bubble_dic, False],
            "smallcaps2": [smallcaps2_dic, False],
            "superscript": [superscript_dic, False],
            "subscript1": [subscript1_dic, False],
            "full_width": [full_width_dic, False],
            "antrophobia": [antrophobia_dic, False],
            "currency": [currency_dic, False],
            "special": [special_dic, False],
            "dirty": [dirty_dic, False],
            "dirty2": [dirty2_dic, False],
            "knight": [knight_dic, False],
            "knight2": [knight2_dic, False],
            "thin2": [thin2_dic, False],
            "tiny": [tiny_dic, False],
            "fancy1": [fancy1_dic, False],
            "fancy2": [fancy2_dic, False],
            "fancy3": [fancy3_dic, False],
            "fancy4": [fancy4_dic, False],
            "fancy5": [fancy5_dic, False],
            "fancy6": [fancy6_dic, False],
            "fancy7": [fancy7_dic, False],
            "fancy8": [fancy8_dic, False],
            "fancy9": [fancy9_dic, False],
            "fancy10": [fancy10_dic, False],
            "fancy11": [fancy11_dic, False],
            "fancy12": [fancy12_dic, False],
            "fancy13": [fancy13_dic, False],
            "fancy14": [fancy14_dic, False],
            "fancy15": [fancy15_dic, False],
            "fancy16": [fancy16_dic, False],
            "fancy17": [fancy17_dic, False],
            "fancy18": [fancy18_dic, False],
            "fancy19": [fancy19_dic, False],
            "fancy20": [fancy20_dic, False],
            "fancy21": [fancy21_dic, False],
            "fancy22": [fancy22_dic, False],
            "fancy23": [fancy23_dic, False],
            "fancy24": [fancy24_dic, False],
            "fancy25": [fancy25_dic, False],
            "fancy26": [fancy26_dic, False],
            "fancy27": [fancy27_dic, False],
            "fancy28": [fancy28_dic, False],
            "fancy29": [fancy29_dic, False],
            "fancy30": [fancy30_dic, False],
            "symbols": [symbols_dic, False],
            "fancy31": [fancy31_dic, False],
            "fancy32": [fancy32_dic, False],
            "fancy33": [fancy33_dic, False],
            "fancy34": [fancy34_dic, False],
            "fancy35": [fancy35_dic, False],
            "fancy36": [fancy36_dic, False],
            "fancy37": [fancy37_dic, False],
            "fancy38": [fancy38_dic, False],
            "fancy39": [fancy39_dic, False],
            "fancy40": [fancy40_dic, False],
            "fancy41": [fancy41_dic, False],
            "fancy42": [fancy42_dic, False],
            "fancy43": [fancy43_dic, False],
            "fancy44": [fancy44_dic, False],
            "fancy45": [fancy45_dic, False],
            "fancy46": [fancy46_dic, False],
            "fancy47": [fancy47_dic, False],
            "fancy48": [fancy48_dic, False],
            "fancy49": [fancy49_dic, False],
            "fancy50": [fancy50_dic, False],
            "fancy51": [fancy51_dic, False],
            "fancy52": [fancy52_dic, False],
            "fancy53": [fancy53_dic, False],
            "fancy54": [fancy54_dic, False],
            "fancy55": [fancy55_dic, False],
            "fancy56": [fancy56_dic, False],
            "fancy57": [fancy57_dic, False],
            "fancy58": [fancy58_dic, False],
            "fancy59": [fancy59_dic, False],
            "fancy60": [fancy60_dic, False],
            "fancy61": [fancy61_dic, False],
            "fancy62": [fancy62_dic, False],
            "fancy63": [fancy63_dic, False],
            "fancy64": [fancy64_dic, False],
            "fancy65": [fancy65_dic, False],
            "fancy66": [fancy66_dic, False],
            "fancy67": [fancy67_dic, False],
            "fancy68": [fancy68_dic, False],
            "fancy69": [fancy69_dic, False],
            "fancy70": [fancy70_dic, False],
            "fancy71": [fancy71_dic, False],
            "fancy72": [fancy72_dic, False],
            "fancy73": [fancy73_dic, False],
            "fancy74": [fancy74_dic, False],
            "fancy75": [fancy75_dic, False],
            "fancy76": [fancy76_dic, False],
            "fancy77": [fancy77_dic, False],
            "fancy78": [fancy78_dic, False],
            "fancy79": [fancy79_dic, False],
            "fancy80": [fancy80_dic, False],
            "fancy81": [fancy81_dic, False],
            "fancy82": [fancy82_dic, False],
            "fancy83": [fancy83_dic, False],
            "fancy84": [fancy84_dic, False],
            "contouring3": [contouring3_dic, False],
            "contouring4": [contouring4_dic, False],
            "love2": [love2_dic, False],
            "fancy85": [fancy85_dic, False],
            "fancy86": [fancy86_dic, False],
            "white_square": [white_square_dic, False],
            "black_square": [black_square_dic, False],
            "paranormal": [paranormal_dic, False],
            "black_bubble": [black_bubble_dic, False],
            "smallcaps3": [smallcaps3_dic, False],
            "fraktur2": [fraktur2_dic, False],
            "bold_fraktur": [bold_fraktur_dic, False],
            "bold_script": [bold_script_dic, False],
            "handwriting1": [handwriting1_dic, False],
            "handwriting2": [handwriting2_dic, False],
            "rusify": [rusify_dic, False],
            "monospace": [monospace_dic, False],
            "h4k3r": [h4k3r_dic, False],
            "thin3": [thin3_dic, False],
            "tiny2": [tiny2_dic, False],
            "upside_down1": [upside_down1_dic, False],
            "chinese_mafia": [chinese_mafia_dic, False],
            "sunday_cuddle": [sunday_cuddle_dic, False],
            "native_lands": [native_lands_dic, False],
            "foreign_friends": [foreign_friends_dic, False],
            "atlantic": [atlantic_dic, False],
            "funky_fresh": [funky_fresh_dic, False],
            "greek_legends": [greek_legends_dic, False],
            "minion": [minion_dic, False],
            "hippie": [hippie_dic, False],
            "sarah": [sarah_dic, False],
            "fancy87": [fancy87_dic, False],
            "fancy88": [fancy88_dic, False],
            "fancy89": [fancy89_dic, False],
            "fancy90": [fancy90_dic, False],
            "fancy91": [fancy91_dic, False],
            "fancy92": [fancy92_dic, False],
            "dwarf": [dwarf_dic, False],
            "lilia": [lilia_dic, False],
            "fancy93": [fancy93_dic, False],
            "fancy94": [fancy94_dic, False],
            "fancy95": [fancy95_dic, False],
            "smooth1": [smooth1_dic, False],
            "fancy96": [fancy96_dic, False],
            "fancy97": [fancy97_dic, False],
            "fancy98": [fancy98_dic, False],
            "fancy99": [fancy99_dic, False],
            "smooth2": [smooth2_dic, False],
            "carrier1": [carrier_dic, False],
            "fancy100": [fancy100_dic, False],
            "carrier2": [carrier2_dic, False],
            "fancy101": [fancy101_dic, False],
            "awesome": [awesome_dic, False],
            "cjk": [cjk_dic, False],
            "fancy102": [fancy102_dic, False],
            "fancy103": [fancy103_dic, False],
            "cute1": [cute_dic, False],
            "cute2": [cute2_dic, False],
            "cute3": [cute3_dic, False],
            "fancy104": [fancy104_dic, False],
            "malayalam": [malayalam_dic, False],
            "fancy105": [fancy105_dic, False],
            "fancy106": [fancy106_dic, False],
            "fancy107": [fancy107_dic, False],
            "cranky": [cranky_dic, False],
            "lopioo": [lopioo_dic, False],
            "scammer": [scammer_dic, False],
            "strikethrough": [strikethrough_dic, False],
            "drako": [drako_dic, False],
            "celtic": [celtic_dic, False],
            "fasion": [fasion_dic, False],
            "curly": [curly_dic, False],
            "russian": [russian_dic, False],
            "russian2": [russian2_dic, False],
            "messletters": [messletters_dic, False],
            "coptic1": [coptic1_dic, False],
            "swirly": [swirly_dic, False],
            "dotted": [dotted_dic, False],
            "coptic2": [coptic2_dic, False],
            "parenthesized": [parenthesized_dic, False],
            "strange": [strange_dic, False],
            "fancy108": [fancy108_dic, False],
            "wiggly": [wiggly_dic, False],
            "hazy": [hazy_dic, False],
            "hyves": [hyves_dic, False],
            "awcute": [awcute_dic, False],
            "slammer": [slammer_dic, False],
            "foxy": [foxy_dic, False],
            "fancy109": [fancy109_dic, False],
            "fancy110": [fancy110_dic, False],
            "fancy111": [fancy111_dic, False],
            "fancy112": [fancy112_dic, False],
            "fancy113": [fancy113_dic, False],
            "fancy114": [fancy114_dic, False],
            "fancy115": [fancy115_dic, False],
            "fancy116": [fancy116_dic, False],
            "fancy117": [fancy117_dic, False],
            "fancy118": [fancy118_dic, False],
            "fancy119": [fancy119_dic, False],
            "fancy120": [fancy120_dic, False],
            "fancy121": [fancy121_dic, False],
            "fancy122": [fancy122_dic, False],
            "fancy123": [fancy123_dic, False],
            "fancy124": [fancy124_dic, False],
            "fancy125": [fancy125_dic, False],
            "fancy126": [fancy126_dic, False],
            "fancy127": [fancy127_dic, False],
            "fancy128": [fancy128_dic, False],
            "fancy129": [fancy129_dic, False],
            "sheqi": [sheqi_dic, False],
            "fari": [fari_dic, False],
            "shasha": [shasha_dic, False],
            "pin1": [pin1_dic, False],
            "pin2": [pin2_dic, False],
            "sign": [sign_dic, False],
            "bud1": [bud1_dic, False],
            "bud2": [bud2_dic, False],
            "old_italic": [old_italic_dic, False],
            "ninja": [ninja_dic, False],
            "tai_viet": [tai_viet_dic, False],
            "subscript2": [subscript2_dic, False],
            "fancy130": [fancy130_dic, False],
            "smooth3": [smooth3_dic, False],
            "flaky": [flaky_dic, False],
            "tiny_caps": [tiny_caps_dic, False],
            "high_above": [high_above_dic, False],
            "handwriting3": [handwriting3_dic, False],
            "fantasy2": [fantasy2_dic, False],
            "manga": [manga_dic, False],
            "upside_down2": [upside_down2_dic, False],
            "cool": [cool_dic, False],
            "instagram": [instagram_dic, False],
            "fancy131": [fancy131_dic, False],
            "fancy132": [fancy132_dic, False],
            "fancy133": [fancy133_dic, False],
            "fancy134": [fancy134_dic, False],
            "fancy135": [fancy135_dic, False],
            "fancy136": [fancy136_dic, False],
            "fancy137": [fancy137_dic, False],
            "fancy138": [fancy138_dic, False],
            "fancy139": [fancy139_dic, False],
            "fancy140": [fancy140_dic, False],
            "fancy141": [fancy141_dic, False],
            "fancy142": [fancy142_dic, False],
            "fancy143": [fancy143_dic, False],
            "fancy144": [fancy144_dic, False],
            "fancy145": [fancy145_dic, False],
            "fancy146": [fancy146_dic, False],
            "fancy147": [fancy147_dic, False],
            "fancy148": [fancy148_dic, False],
            "tarty1": [tarty1_dic, False],
            "tarty2": [tarty2_dic, False],
            "tarty3": [tarty3_dic, False],
            "tarty4": [tarty4_dic, False],
            "tarty5": [tarty5_dic, False],
            "tarty6": [tarty6_dic, False],
            "tarty7": [tarty7_dic, False],
            "tarty8": [tarty8_dic, False],
            "tarty9": [tarty9_dic, False],
            "got": [got_dic, False],
            "vip": [vip_dic, False],
            "crazy1": [crazy1_dic, False],
            "crazy2": [crazy2_dic, False],
            "crazy3": [crazy3_dic, False],
            "cruse": [cruse_dic, False],
            "drippy": [drippy_dic, False],
            "stylish": [stylish_dic, False],
            "ayda": [ayda_dic, False],
            "small_fancy": [small_fancy_dic, False],
            "boom": [boom_dic, False],
            "cross_jesus": [cross_jesus_dic, False],
            "dark_tattoo": [dark_tattoo_dic, False],
            "lucifer": [lucifer_dic, False],
            "ascii_roman": [ascii_roman_dic, False],
            "squiggle1": [squiggle1_dic, False],
            "squiggle2": [squiggle2_dic, False],
            "vaporwave": [vaporwave_dic, False],
            "emoji": [emoji_dic, False]
            }

DECORATIONS_MAP = {"angry1": angry1,  # pragma: no cover
                   "arrow1": arrow1,
                   "arrow2": arrow2,
                   "arrow3": arrow3,
                   "arrow4": arrow4,
                   "arrow5": arrow5,
                   "arrow6": arrow6,
                   "arrow7": arrow7,
                   "arrow8": arrow8,
                   "arrow_wave1": arrow_wave1,
                   "arrow_wave2": arrow_wave2,
                   "ball1": ball1,
                   "ball2": ball2,
                   "ball3": ball3,
                   "barcode1": barcode1,
                   "bazar1": bazar1,
                   "block1": block1,
                   "block2": block2,
                   "bow1": bow1,
                   "bubble": bubble,
                   "cat1": cat1,
                   "cat2": cat2,
                   "cat3": cat3,
                   "cell1": cell1,
                   "champion1": champion1,
                   "chess1": chess1,
                   "confused1": confused1,
                   "confused2": confused2,
                   "cross1": cross1,
                   "depressed": depressed,
                   "diamon3": diamon3,
                   "diamond1": diamond1,
                   "diamond2": diamond2,
                   "egypt1": egypt1,
                   "emotions1": emotions1,
                   "fancy1": fancy1,
                   "fancy2": fancy2,
                   "fancy3": fancy3,
                   "fancy4": fancy4,
                   "fancy5": fancy5,
                   "fancy6": fancy6,
                   "fancy7": fancy7,
                   "fancy8": fancy8,
                   "fancy9": fancy9,
                   "fancy10": fancy10,
                   "fancy11": fancy11,
                   "fancy12": fancy12,
                   "fancy13": fancy13,
                   "fancy14": fancy14,
                   "fancy15": fancy15,
                   "fancy16": fancy16,
                   "fancy17": fancy17,
                   "fancy18": fancy18,
                   "fancy19": fancy19,
                   "fancy20": fancy20,
                   "fancy21": fancy21,
                   "fancy22": fancy22,
                   "fancy23": fancy23,
                   "fancy24": fancy24,
                   "fancy25": fancy25,
                   "fancy26": fancy26,
                   "fancy27": fancy27,
                   "fancy28": fancy28,
                   "fancy29": fancy29,
                   "fancy30": fancy30,
                   "fancy31": fancy31,
                   "fancy32": fancy32,
                   "fancy33": fancy33,
                   "fancy34": fancy34,
                   "fancy35": fancy35,
                   "fancy36": fancy36,
                   "fancy37": fancy37,
                   "fancy38": fancy38,
                   "fancy39": fancy39,
                   "fancy40": fancy40,
                   "fancy41": fancy41,
                   "fancy42": fancy42,
                   "fancy43": fancy43,
                   "fancy44": fancy44,
                   "fancy45": fancy45,
                   "fancy46": fancy46,
                   "fancy47": fancy47,
                   "fancy48": fancy48,
                   "fancy49": fancy49,
                   "fancy50": fancy50,
                   "fancy51": fancy51,
                   "fancy52": fancy52,
                   "fancy53": fancy53,
                   "fancy54": fancy54,
                   "fancy55": fancy55,
                   "fancy56": fancy56,
                   "fancy57": fancy57,
                   "fancy58": fancy58,
                   "fancy59": fancy59,
                   "fancy60": fancy60,
                   "fancy61": fancy61,
                   "fancy62": fancy62,
                   "fancy63": fancy63,
                   "fancy64": fancy64,
                   "fancy65": fancy65,
                   "fancy66": fancy66,
                   "fancy67": fancy67,
                   "fancy68": fancy68,
                   "fancy69": fancy69,
                   "fancy70": fancy70,
                   "fancy71": fancy71,
                   "fancy72": fancy72,
                   "fancy73": fancy73,
                   "fancy74": fancy74,
                   "fancy75": fancy75,
                   "fancy76": fancy76,
                   "fancy77": fancy77,
                   "fancy78": fancy78,
                   "fancy79": fancy79,
                   "fancy80": fancy80,
                   "fancy81": fancy81,
                   "fancy82": fancy82,
                   "fancy83": fancy83,
                   "fancy84": fancy84,
                   "fancy85": fancy85,
                   "fancy86": fancy86,
                   "fancy87": fancy87,
                   "fancy88": fancy88,
                   "fancy89": fancy89,
                   "fancy90": fancy90,
                   "fancy91": fancy91,
                   "fancy92": fancy92,
                   "fancy93": fancy93,
                   "fancy94": fancy94,
                   "fancy95": fancy95,
                   "flame1": flame1,
                   "flower1": flower1,
                   "food1": food1,
                   "food2": food2,
                   "haha": haha,
                   "happy1": happy1,
                   "happy_new_year": happy_new_year,
                   "hawaii1": hawaii1,
                   "hawaii2": hawaii2,
                   "heart1": heart1,
                   "heart2": heart2,
                   "heart3": heart3,
                   "heart4": heart4,
                   "heart5": heart5,
                   "heart6": heart6,
                   "heart7": heart7,
                   "heart8": heart8,
                   "heart9": heart9,
                   "heart10": heart10,
                   "heart11": heart11,
                   "heart12": heart12,
                   "heart13": heart13,
                   "heart14": heart14,
                   "heart15": heart15,
                   "line1": line1,
                   "line2": line2,
                   "line3": line3,
                   "line4": line4,
                   "love_music": love_music,
                   "lucky1": lucky1,
                   "missile1": missile1,
                   "mountain1": mountain1,
                   "mountain2": mountain2,
                   "mountain3": mountain3,
                   "music1": music1,
                   "music2": music2,
                   "music3": music3,
                   "pencil1": pencil1,
                   "poker1": poker1,
                   "puzzle1": puzzle1,
                   "puzzle2": puzzle2,
                   "puzzle3": puzzle3,
                   "sad1": sad1,
                   "sad2": sad2,
                   "sad3": sad3,
                   "sad4": sad4,
                   "smile1": smile1,
                   "snow1": snow1,
                   "snow2": snow2,
                   "soccer1": soccer1,
                   "star1": star1,
                   "star2": star2,
                   "star3": star3,
                   "star4": star4,
                   "star5": star5,
                   "star6": star6,
                   "star7": star7,
                   "star8": star8,
                   "star9": star9,
                   "star10": star10,
                   "star11": star11,
                   "star12": star12,
                   "star13": star13,
                   "star14": star14,
                   "star15": star15,
                   "star16": star16,
                   "star17": star17,
                   "star18": star18,
                   "star19": star19,
                   "star20": star20,
                   "star21": star21,
                   "star22": star22,
                   "star23": star23,
                   "star24": star24,
                   "star25": star25,
                   "temple1": temple1,
                   "title1": title1,
                   "tree1": tree1,
                   "wall1": wall1,
                   "wave1": wave1,
                   "wave2": wave2,
                   "wave3": wave3,
                   "wave4": wave4,
                   "wave5": wave5,
                   "wave6": wave6,
                   "wave7": wave7,
                   "wave8": wave8,
                   "wave9": wave9}

FONT_NAMES = sorted(FONT_MAP)  # pragma: no cover
ASCII_FONTS = sorted(set(FONT_NAMES) - set(NON_ASCII_FONTS))
DECORATION_NAMES = sorted(DECORATIONS_MAP)  # pragma: no cover
ART_NAMES = sorted(art_dic)  # pragma: no cover
ASCII_ARTS = sorted(set(ART_NAMES) - set(NON_ASCII_ARTS))
FONT_COUNTER = len(FONT_NAMES)  # pragma: no cover
ART_COUNTER = len(ART_NAMES)  # pragma: no cover
DECORATION_COUNTER = len(DECORATION_NAMES)  # pragma: no cover
DEFAULT_FONT = "standard"  # pragma: no cover
